unit RegIDEIntf;

{$mode objfpc}{$H+}

interface

uses
  ProjectIntf, NewItemIntf, LazIDEIntf, Controls, Forms;

type

  { TCustomBCGIProjectDescriptor }

  TCustomBCGIProjectDescriptor = class(TProjectDescriptor)
  protected
    class function SourceText: string; virtual; abstract;
  public
    constructor Create; override;
    function CreateStartFiles({%H-}AProject: TLazProject): TModalResult; override;
    function InitProject(AProject: TLazProject): TModalResult; override;
  end;

  { TBCGIProjectDescriptor }

  TBCGIProjectDescriptor = class(TCustomBCGIProjectDescriptor)
  protected
    class function SourceText: string; override;
  public
    function CreateStartFiles({%H-}AProject: TLazProject): TModalResult; override;
    function GetLocalizedName: string; override;
    function GetLocalizedDescription: string; override;
  end;

  { THelloWorldFileDescPascalUnit }

  THelloWorldFileDescPascalUnit = class(TFileDescPascalUnit)
  public
    constructor Create; override;
    function CreateSource({%H-}const FileName,{%H-}SourceName,
      {%H-}ResourceName: string): string; override;
  end;

  { TBasicPluginsFileDescPascalUnit }

  TBasicPluginsFileDescPascalUnit = class(TFileDescPascalUnit)
  public
    constructor Create; override;
    function CreateSource({%H-}const FileName,{%H-}SourceName,
      {%H-}ResourceName: string): string; override;
  end;

const
  le = LineEnding;
  SCGIAppName = 'CGI application';
  SCGIAppDesc = 'Create a (Brook)CGI application using B-CGI plugin.';
  SIDEItemCategoryName = 'B-CGI';
  SHelloWorldName = 'Hello';
  SPluginsName = 'Plugins';

procedure Register;

implementation

procedure Register;
begin
  RegisterNewItemCategory(TNewIDEItemCategory.Create(SIDEItemCategoryName));
  RegisterProjectDescriptor(TBCGIProjectDescriptor.Create, SIDEItemCategoryName);
  RegisterProjectFileDescriptor(THelloWorldFileDescPascalUnit.Create,
    '', niifInherited);
  RegisterProjectFileDescriptor(TBasicPluginsFileDescPascalUnit.Create,
    '', niifInherited);
end;

{ TCustomBCGIProjectDescriptor }

constructor TCustomBCGIProjectDescriptor.Create;
begin
  inherited Create;
  Flags := Flags - [pfMainUnitHasCreateFormStatements, pfRunnable];
  Name := SCGIAppName;
end;

function TCustomBCGIProjectDescriptor.CreateStartFiles(AProject: TLazProject
  ): TModalResult;
begin
  Result := mrOK;
end;

function TCustomBCGIProjectDescriptor.InitProject(AProject: TLazProject
  ): TModalResult;
var
  VProject: TLazProjectFile;
begin
  Result := inherited InitProject(AProject);
  VProject := AProject.CreateProjectFile('cgi1.lpr');
  VProject.IsPartOfProject := True;
  AProject.AddFile(VProject, False);
  AProject.AddPackageDependency('BrookRT');
  AProject.AddPackageDependency('BCGIPL');
  AProject.LazCompilerOptions.Win32GraphicApp := False;
  AProject.LazCompilerOptions.LinkSmart := True;
  AProject.LazCompilerOptions.OptimizationLevel := 2;
  AProject.LazCompilerOptions.TargetFileNameApplyConventions := False;
  AProject.LazCompilerOptions.UseExternalDbgSyms := True;
  AProject.LazCompilerOptions.UseLineInfoUnit := False;
  AProject.SessionStorage := pssInProjectInfo;
  AProject.MainFileID := 0;
  AProject.Title := 'cgi1';
  AProject.MainFile.SetSourceText(SourceText);
end;

{ TBCGIProjectDescriptor }

function TBCGIProjectDescriptor.GetLocalizedName: string;
begin
  Result := SCGIAppName;
end;

function TBCGIProjectDescriptor.GetLocalizedDescription: string;
begin
  Result := SCGIAppDesc;
end;

class function TBCGIProjectDescriptor.SourceText: string;
begin
  Result :=
    'program cgi1;'+le+
    ''+le+
    '{$mode objfpc}{$H+}'+le+
    ''+le+
    'uses'+le+
    '  BrookApplication, Plugins, Hello;'+le+
    ''+le+
    'begin'+le+
    '  BrookApp.Run;'+le+
    'end.';
end;

function TBCGIProjectDescriptor.CreateStartFiles(AProject: TLazProject
  ): TModalResult;
var
  VFileDesc: TProjectFileDescriptor;
begin
  VFileDesc := ProjectFileDescriptors.FindByName(SHelloWorldName);
  LazarusIDE.DoNewEditorFile(VFileDesc, '', '',
    [nfIsPartOfProject, nfOpenInEditor, nfCreateDefaultSrc]);
  VFileDesc := ProjectFileDescriptors.FindByName(SPluginsName);
  LazarusIDE.DoNewEditorFile(VFileDesc, '', '',
    [nfIsPartOfProject, nfOpenInEditor, nfCreateDefaultSrc]);
  Result := mrOK;
end;

{ THelloWorldFileDescPascalUnit }

constructor THelloWorldFileDescPascalUnit.Create;
begin
  inherited Create;
  Name := SHelloWorldName;
  DefaultFileName := 'hello.pas';
  DefaultSourceName := SHelloWorldName;
end;

function THelloWorldFileDescPascalUnit.CreateSource(const FileName,
  SourceName, ResourceName: string): string;
begin
  Result :=
    'unit Hello;'+le+
    ''+le+
    '{$mode objfpc}{$H+}'+le+
    ''+le+
    'interface'+le+
    ''+le+
    'uses'+le+
    '  BrookAction;'+le+
    ''+le+
    'type'+le+
    '  THello = class(TBrookAction)'+le+
    '  public'+le+
    '    procedure Request({%H-}ARequest: TBrookRequest;'+le+
    '      {%H-}AResponse: TBrookResponse); override;'+le+
    '  end;'+le+
    ''+le+
    'implementation'+le+
    ''+le+
    'procedure THello.Request(ARequest: TBrookRequest; AResponse: TBrookResponse);'+le+
    'begin'+le+
    '  Echo(''Hello world!'');'+le+
    'end;'+le+
    ''+le+
    'initialization'+le+
    '  // Open "localhost/cgi-bin/cgi/hello" in your browser.'+le+
    '  THello.Register(''/hello/'');'+le+
    ''+le+
    'end.';
end;

{ TBasicPluginsFileDescPascalUnit }

constructor TBasicPluginsFileDescPascalUnit.Create;
begin
  inherited Create;
  Name := SPluginsName;
  DefaultFileName := 'plugins.pas';
  DefaultSourceName := SPluginsName;
end;

function TBasicPluginsFileDescPascalUnit.CreateSource(const FileName,
  SourceName, ResourceName: string): string;
begin
  Result :=
    'unit Plugins;'+le+
    ''+le+
    '{$mode objfpc}{$H+}'+le+
    ''+le+
    'interface'+le+
    ''+le+
    'uses'+le+
    '  BCGI;'+le+
    ''+le+
    'implementation'+le+
    ''+le+
    'end.';
end;

end.
